# map-1905

[map-1905](https://dev.iopsys.eu/iopsys/map-1905)

##  Introduction

MAP stands for multi-AP and it provides a set of libraries and plugin to understand/translate multi-AP based messages. Map-1905 is capable of handling both profile-1 and profile-2 type of multi-AP messages.

## Overview

The Map-1905 packages is based on the WiFi-alliances multiAP specification [Wi-Fi CERTIFIED EasyMesh](https://www.wi-fi.org/discover-wi-fi/wi-fi-easymesh).

It provides one plugin:
- `libwifimap2.so`

and an utility library:
- `libmaputils.so`

This package does not provide the complete MAP stack. It can only handle the following:

MultiAP stack uses ieee1905 specification for information sharing and as per ieee1905 specification each message needs to be send in a ethernet frame with ethertype 0x893A and it must consists CMDU header(Table 6-3 of ieee1905 specification), which further consists list of TLVs.

Now, as per the specification each ethernet frame can only transport a payload of 1500 bytes, in case the map payload is more than 1500 it needs to follow rules of fragmentation as specified in section 7.1 in ieee1905 specification.

Further if we see, the information present in the TLVs sometime contains information in bits and sometimes it has information in zero or more group format.

So, this library provides a way to easily translate ethernet frames to list of tlv streams(as each frame can have more that one TLV), each stream corresponds to one specific type of tlv.

It also provide the API to translate the tlv stream to C structure, so that the map-modules directly works on them and vice-versa.

This plugin provided by this package is meant to run using the 1905 stack's plugin manager, as map stack has to use the 1905 stack for transport.

On successful invocation it provides two major functionality:

1. Map module registration
2. Functionality to send MultiAP CMDUs

On the other hand, the maputils library provides C structures for CMDUs and TLVs along with API to translate/work on map

1. ethernet frames to C structure with list of TLV streams
2. TLV streams to C specific TLV structures, and vise-versa

### Ubus API

This is a verbose print of all methods published to ubus on a device with map enabled 1905 stack:

```bash
'map.1905' @708d34e5
        "register":{"module":"Integer","data":"String"}
        "send":{"type":"Integer","egress":"String","destination":"String","mid":"Integer","data":"String"}
        "send_array":{"type":"Integer","egress":"String","destination":"String","mid":"Integer","tlv_array":"Array"}
```

- For more info on the `map.1905` ubus API see [link](./docs/api/map.1905.md)

### Map module registration

On successful registration, map-plugin broadcast an ubus event on arrival of multiAP specific CMDUs in below format.

```bash
{ "ieee1905.cmdu": {
                    "type":0,
                    "mid":1234,
                    "ingress":"br-lan",
                    "source":"02:22:07:9f:b7:fe",
                    "tlvs":"0100060222079fb7fe0200060022079fb7fc"
                   }
}
```

### Functionality to send MAP CMDUs

After registration, map-modules can use the `send` or `send_array` method of `map.1905` object to send any map/ieee1905 specific cmdu to other nodes.

Send api doesn't validate the payload before sending the CMDU, it is responsibility of map-module to validate/sanitize(i.e. check if the CMDU has all the required TLVs) the CMDU before using the `send` method. CMDU stream can be validated using `map_validate_cmdu_stream` API of libmaputils.

In the below example:

 * type: cmdu type number as per the map/ieee1905 specification
 * egress:  Interface name from which data needs to be sent out
 * destination: Destination node's AL mac
 * mid: Message id as decribed in ieee1905 specification, for query/notification messages this value will be ignored and re-generated by ieee1905 stack.
 * data: Hex-string of tlv streams, all combined in one, "end of message" tlv not required to be included in the stream.
 * tlv_array: Array of hex-strings each containing one tlv stream, "end of message" tlv not required to be include in the streams.

<u>Example of sending cmdu with no TLV:</u>
```bash
root@iopsys:~# ubus call map.1905 send '{
        "type":2,
        "egress":"br-lan",
        "destination":"02:22:07:9f:b7:ff"
}'

root@iopsys:~# ubus call map.1905 send '{
        "type":2,
        "egress":"br-lan",
        "destination":"02:22:07:9f:b7:ff"
}'
```

<u>Example of sending cmdu with TLVs:</u>
```bash
root@iopsys:~# ubus call map.1905 send '{
        "type":7,
        "egress":"br-lan",
        "destination":"02:22:07:9f:b7:ff",
        "data":"010006022207765c570d0001000e000100
}'

root@iopsys:~# ubus call map.1905 send_array '{
        "type":7,
        "egress":"br-lan",
        "destination":"02:22:07:9f:b7:ff",
        "tlv_array":["010006022207765c57","0d000100","0e000100"]
}'

```

## Integration Note

Map plugin has few dependencies on 1905-stack, which are clubbed in few APIs as described in [link](./src/map_module.h)

- For more info on CMDU and TLVs related API and C-structures see [link](./include/map2.h) [link2] (./include/maputils.h)
- For more info on map-module registration structures see [link](./include/map_module.h)

## Dependencies ##

To successfully build map-1905, the following libraries are needed:

| Dependency        | Link                                       			| License        |
| ----------------- | ---------------------------------------------------------------- 	| -------------- |
| libubox    	    | https://git.openwrt.org/project/libubox.git 			| BSD            |
| libubus     	    | https://git.openwrt.org/project/ubus.git    			| LGPL 2.1       |
| libjson-c   	    | https://s3.amazonaws.com/json-c_releases    			| MIT            |

controller/-/merge_requests/new?merge_request%5Bsource_branch%5D=docs&merge_request%5Bsource_project_id%5D=254&merge_request%5Btarget_branch%5D=devel&merge_request%5Btarget_project_id%5D=254