# map.1905 Schema

```
https://www.iopsys.eu/usp.raw.json
```

| Custom Properties | Additional Properties |
| ----------------- | --------------------- |
| Forbidden         | Forbidden             |

# map.1905

| List of Methods           |
| ------------------------- |
| [register](#register)     | Method | map.1905 (this schema) |
| [send](#send)             | Method | map.1905 (this schema) |
| [send_array](#send_array) | Method | map.1905 (this schema) |

## register

### Register with map plugin

`register`

- type: `Method`

### register Type

`object` with following properties:

| Property | Type   | Required     |
| -------- | ------ | ------------ |
| `input`  | object | **Required** |
| `output` | object | Optional     |

#### input

`input`

- is **required**
- type: `object`

##### input Type

`object` with following properties:

| Property | Type    | Required |
| -------- | ------- | -------- |
| `data`   | string  | Optional |
| `module` | integer | Optional |

#### data

`data`

- is optional
- type: reference

##### data Type

`string`

All instances must conform to this regular expression (test examples
[here](<https://regexr.com/?expression=(%5B0-9a-fA-F%5D%5B0-9a-fA-F%5D)%2B>)):

```regex
([0-9a-fA-F][0-9a-fA-F])+
```

#### module

`module`

- is optional
- type: `integer`

##### module Type

`integer`

### Ubus CLI Example

```
ubus call map.1905 register {"module":10545443,"data":"F0aDB4D2"}
```

### JSONRPC Example

```json
{
  "jsonrpc": "2.0",
  "id": 0,
  "method": "call",
  "params": ["<SID>", "map.1905", "register", { "module": 10545443, "data": "F0aDB4D2" }]
}
```

#### output

`output`

- is optional
- type: `object`

##### output Type

`object` with following properties:

| Property | Type | Required |
| -------- | ---- | -------- |
| None     | None | None     |

### Output Example

```json
{}
```

## send

### Send map cmdu from map plugin

`send`

- type: `Method`

### send Type

`object` with following properties:

| Property | Type   | Required     |
| -------- | ------ | ------------ |
| `input`  | object | **Required** |
| `output` | object | **Required** |

#### input

`input`

- is **required**
- type: `object`

##### input Type

`object` with following properties:

| Property      | Type    | Required |
| ------------- | ------- | -------- |
| `data`        | string  | Optional |
| `dst_macaddr` | string  | Optional |
| `egress`      | string  | Optional |
| `mid`         | integer | Optional |
| `type`        | integer | Optional |

#### data

`data`

- is optional
- type: reference

##### data Type

`string`

All instances must conform to this regular expression (test examples
[here](<https://regexr.com/?expression=(%5B0-9a-fA-F%5D%5B0-9a-fA-F%5D)%2B>)):

```regex
([0-9a-fA-F][0-9a-fA-F])+
```

#### dst_macaddr

mac address in string format

`dst_macaddr`

- is optional
- type: reference

##### dst_macaddr Type

`string`

- minimum length: 17 characters
- maximum length: 17 characters All instances must conform to this regular expression (test examples
  [here](<https://regexr.com/?expression=(%5B0-9a-fA-F%5D%7B2%7D%3A)%7B5%7D%5B0-9a-fA-F%5D%7B2%7D>)):

```regex
([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}
```

#### egress

Linux network interface name

`egress`

- is optional
- type: reference

##### egress Type

`string`

- minimum length: 2 characters
- maximum length: 256 characters All instances must conform to this regular expression

```regex
([a-z]+|br-){1}[a-z]+[0-9]
```

- test example: [eth0](<https://regexr.com/?expression=(%5Ba-z%5D%2B%7Cbr-)%7B1%7D%5Ba-z%5D%2B%5B0-9%5D&text=eth0>)
- test example: [br-lan](<https://regexr.com/?expression=(%5Ba-z%5D%2B%7Cbr-)%7B1%7D%5Ba-z%5D%2B%5B0-9%5D&text=br-lan>)

##### egress Examples

```json
eth0
```

```json
br - lan
```

#### mid

CMDU message identifier

`mid`

- is optional
- type: reference

##### mid Type

`integer`

- minimum value: `0`
- maximum value: `65535`

#### type

CMDU message type

`type`

- is optional
- type: reference

##### type Type

`integer`

- minimum value: `1`
- maximum value: `32819`

### Ubus CLI Example

```
ubus call map.1905 send {"type":1417,"egress":"hvvvolrxgvjr8","dst_macaddr":"BF:ED:e4:B5:2c:03","mid":31829,"data":"C3"}
```

### JSONRPC Example

```json
{
  "jsonrpc": "2.0",
  "id": 0,
  "method": "call",
  "params": [
    "<SID>",
    "map.1905",
    "send",
    { "type": 1417, "egress": "hvvvolrxgvjr8", "dst_macaddr": "BF:ED:e4:B5:2c:03", "mid": 31829, "data": "C3" }
  ]
}
```

#### output

`output`

- is **required**
- type: `object`

##### output Type

`object` with following properties:

| Property | Type    | Required |
| -------- | ------- | -------- |
| `mid`    | integer | Optional |

#### mid

`mid`

- is optional
- type: `integer`

##### mid Type

`integer`

- minimum value: `0`

### Output Example

```json
{ "mid": 78063656 }
```

## send_array

### Send map cmdu from map plugin

send tlv array

`send_array`

- type: `Method`

### send_array Type

`object` with following properties:

| Property | Type   | Required     |
| -------- | ------ | ------------ |
| `input`  | object | **Required** |
| `output` | object | **Required** |

#### input

`input`

- is **required**
- type: `object`

##### input Type

`object` with following properties:

| Property      | Type    | Required |
| ------------- | ------- | -------- |
| `dst_macaddr` | string  | Optional |
| `egress`      | string  | Optional |
| `mid`         | integer | Optional |
| `tlv_array`   | array   | Optional |
| `type`        | integer | Optional |

#### dst_macaddr

mac address in string format

`dst_macaddr`

- is optional
- type: reference

##### dst_macaddr Type

`string`

- minimum length: 17 characters
- maximum length: 17 characters All instances must conform to this regular expression (test examples
  [here](<https://regexr.com/?expression=(%5B0-9a-fA-F%5D%7B2%7D%3A)%7B5%7D%5B0-9a-fA-F%5D%7B2%7D>)):

```regex
([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}
```

#### egress

Linux network interface name

`egress`

- is optional
- type: reference

##### egress Type

`string`

- minimum length: 2 characters
- maximum length: 256 characters All instances must conform to this regular expression

```regex
([a-z]+|br-){1}[a-z]+[0-9]
```

- test example: [eth0](<https://regexr.com/?expression=(%5Ba-z%5D%2B%7Cbr-)%7B1%7D%5Ba-z%5D%2B%5B0-9%5D&text=eth0>)
- test example: [br-lan](<https://regexr.com/?expression=(%5Ba-z%5D%2B%7Cbr-)%7B1%7D%5Ba-z%5D%2B%5B0-9%5D&text=br-lan>)

##### egress Examples

```json
eth0
```

```json
br - lan
```

#### mid

CMDU message identifier

`mid`

- is optional
- type: reference

##### mid Type

`integer`

- minimum value: `0`
- maximum value: `65535`

#### tlv_array

`tlv_array`

- is optional
- type: reference

##### tlv_array Type

Array type: reference

All items must be of the type: `string`

All instances must conform to this regular expression (test examples
[here](<https://regexr.com/?expression=(%5B0-9a-fA-F%5D%5B0-9a-fA-F%5D)%2B>)):

```regex
([0-9a-fA-F][0-9a-fA-F])+
```

#### type

CMDU message type

`type`

- is optional
- type: reference

##### type Type

`integer`

- minimum value: `1`
- maximum value: `32819`

### Ubus CLI Example

```
ubus call map.1905 send_array {"type":25798,"egress":"hhfcjghdmlebsn2","dst_macaddr":"Bc:E3:1A:10:dd:9A","mid":62280,"tlv_array":["6872559085","aB","b0cddE64","F8","149babFD"]}
```

### JSONRPC Example

```json
{
  "jsonrpc": "2.0",
  "id": 0,
  "method": "call",
  "params": [
    "<SID>",
    "map.1905",
    "send_array",
    {
      "type": 25798,
      "egress": "hhfcjghdmlebsn2",
      "dst_macaddr": "Bc:E3:1A:10:dd:9A",
      "mid": 62280,
      "tlv_array": ["6872559085", "aB", "b0cddE64", "F8", "149babFD"]
    }
  ]
}
```

#### output

`output`

- is **required**
- type: `object`

##### output Type

`object` with following properties:

| Property | Type    | Required |
| -------- | ------- | -------- |
| `mid`    | integer | Optional |

#### mid

`mid`

- is optional
- type: `integer`

##### mid Type

`integer`

- minimum value: `0`

### Output Example

```json
{ "mid": 6271343 }
```
