#include "map2.h"
#include "maputils.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <json-c/json.h>
#include "map_common.h"

static void dump_hex_bytes(uint8_t *p, uint16_t len)
{
	for (size_t i = 0; i < len; i++) {
		printf("%02x", p[i]);
	}
}

static void dump_tlv_bytes(uint8_t *tlv, uint16_t tlv_len)
{
	if (tlv == NULL)
		return;

	printf("### Dumping TLV(0x%x:%s) len(%d) stream ###\n", *tlv,
	       map_stringify_tlv_type(*tlv), tlv_len);

	dump_hex_bytes(tlv, tlv_len);
	printf("\n### Done ###\n");
}

void test_using_api(void)
{
	printf("## dumping info ##\n");
	map_dump_cmdu_validator();
	printf("## dumping info done ##\n");
}

void hwaddr_aton(char *mac_ascii, uint8_t *mac)
{
	int addr[6];
	if (mac_ascii == NULL)
		return;

	if (sscanf(mac_ascii, "%02x:%02x:%02x:%02x:%02x:%02x",
		   &addr[0], &addr[1], &addr[2], &addr[3],
		   &addr[4], &addr[5]) != EOF) {

		mac[0] = addr[0] & 0xFF;
		mac[1] = addr[1] & 0xFF;
		mac[2] = addr[2] & 0xFF;
		mac[3] = addr[3] & 0xFF;
		mac[4] = addr[4] & 0xFF;
		mac[5] = addr[5] & 0xFF;
	}


}

void test_converting_tlv_cstructure_to_stream(void)
{
	struct tlv_al_mac tlv;
	uint8_t *tlv_stream;
	uint16_t tlv_len = 0;
	size_t i;

	// fill the tlv with correct info
	tlv.tlv_type = TLV_TYPE_AL_MAC_ADDRESS_TYPE;
	tlv.al_mac_address[0] = 0x11;
	tlv.al_mac_address[1] = 0x22;
	tlv.al_mac_address[2] = 0x33;
	tlv.al_mac_address[3] = 0x44;
	tlv.al_mac_address[4] = 0x55;
	tlv.al_mac_address[5] = 0x66;

	tlv_stream = map_put_tlv_cstruct((uint8_t *) &tlv, &tlv_len);

	dump_tlv_bytes(tlv_stream, tlv_len);

	free(tlv_stream);
}

void prepare_and_send_cmdu_cstruct(struct cmdu_cstruct *cmdu_data)
{
	struct json_object *jroot, *jtype, *jmid, *jingress, *jsource, *jtlvs;
	size_t i;
	char tlv_data[3000] = { 0 };

	for (i = 0; i < cmdu_data->num_tlvs; i++) {
		uint16_t len = 0;
		char *tlv_str;
		uint8_t *ss = map_put_tlv_cstruct(cmdu_data->tlvs[i], &len);
		if (ss) {
			tlv_str = _btostr(ss, len);
			strcat(tlv_data, tlv_str);
			free(tlv_str);
			free(ss);
		}
	}

	jroot = json_object_new_object();
	jtype = json_object_new_int(cmdu_data->message_type);
	jmid = json_object_new_int(cmdu_data->message_id);
	jingress = json_object_new_string(cmdu_data->intf_name);
	jsource = json_object_new_string("11:22:33:44:55:66");
	jtlvs = json_object_new_string(tlv_data);

	json_object_object_add(jroot, "type", jtype);
	json_object_object_add(jroot, "mid", jmid);
	json_object_object_add(jroot, "ingress", jingress);
	json_object_object_add(jroot, "source", jsource);
	json_object_object_add(jroot, "tlvs", jtlvs);

	char *json_str = strdup(json_object_to_json_string(jroot));
	json_object_put(jroot);

	printf("JSON(%s)\n", json_str);
	free(json_str);
	printf("\n\n");
}

static void prepare_topology_response(struct cmdu_cstruct *rec_cmdu)
{
	char radio_id[18] = "11:12:13:14:15:16";
	char bssid[18] = "21:22:23:24:25:26";
	char sta[18] = "31:32:33:34:35:36";
	uint32_t i, j;

	struct cmdu_cstruct *cmdu_data;
	cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));

	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_TYPE_TOPOLOGY_RESPONSE;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	/* AP Operational BSS TLV */
	struct tlv_ap_oper_bss *p = (struct tlv_ap_oper_bss *)calloc(1, sizeof(struct tlv_ap_oper_bss));
	p->tlv_type = MAP_TLV_AP_OPERATIONAL_BSS;
	p->radios_nr = 1;
	p->radio = calloc(p->radios_nr, sizeof(*p->radio));
	for(i=0; i<p->radios_nr; i++) {
		hwaddr_aton(radio_id, p->radio[i].radio_id);
		p->radio[i].bss_nr = 1;
		p->radio[i].bss = calloc(p->radio[i].bss_nr, sizeof(*(p->radio[i].bss)));
		for(j=0; j<p->radio[i].bss_nr; j++) {
			hwaddr_aton(bssid, p->radio[i].bss[j].bssid);
			strcpy(p->radio[i].bss[j].ssid, "iopsys_test");
			p->radio[i].bss[j].ssid_len = strlen(p->radio[i].bss[j].ssid);
		}
	}

	/* Associated Client TLV */
	struct tlv_assoc_client *p1 = (struct tlv_assoc_client *)calloc(1, sizeof(struct tlv_assoc_client));
	p1->tlv_type = MAP_TLV_ASSOCIATED_CLIENTS;
	p1->bss_nr = 1;
	p1->bss = calloc(p1->bss_nr, sizeof(*p1->bss));
	for(i=0; i<p1->bss_nr; i++) {
		hwaddr_aton(bssid, p1->bss[i].bssid);
		p1->bss[i].assoc_clients_nr = 1;
		p1->bss[i].clients = calloc(p1->bss[i].assoc_clients_nr, sizeof(*(p1->bss[i].clients)));
		for(j=0; j<p1->bss[i].assoc_clients_nr; j++) {
			hwaddr_aton(sta, p1->bss[i].clients[j].client_addr);
			p1->bss[i].clients[j].uptime = 1500;
		}
	}

	cmdu_data->num_tlvs = 2;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	cmdu_data->tlvs[0] = (uint8_t *)p;
	cmdu_data->tlvs[1] = (uint8_t *)p1;

        prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

static void prepare_ap_capability_response(struct cmdu_cstruct *rec_cmdu)
{
	struct cmdu_cstruct *cmdu_data;
	cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));
	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_AP_CAPABILITY_REPORT;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	/* AP basic Radio Capability TLV*/
	struct tlv_ap_radio_basic_cap *p = (struct tlv_ap_radio_basic_cap *)calloc(1, sizeof(struct tlv_ap_radio_basic_cap));
	p->tlv_type = MAP_TLV_AP_RADIO_BASIC_CAPABILITIES;
	char radio_id[18] = "11:12:13:14:15:16";
	hwaddr_aton(radio_id, p->radio_id);
	p->max_bss_nr = 1;
	p->operating_classes_nr = 5;
	uint8_t j;
	p->operating_class = calloc(p->operating_classes_nr, sizeof(*p->operating_class));
	for(j=0; j<p->operating_classes_nr; j++) {
		p->operating_class[j].op_class = 121;
		p->operating_class[j].max_tx_power = 122;
		p->operating_class[j].non_op_ch_nr = 0;
		if (p->operating_class[j].non_op_ch_nr > 0) {
			p->operating_class[j].channel = (uint8_t *)calloc(p->operating_class[j].non_op_ch_nr, sizeof(uint8_t));
			if (p->operating_class[j].channel) {
				uint8_t k;
				for(k=0; j<p->operating_class[j].non_op_ch_nr; k++) {
					p->operating_class[j].channel[k] = 'a' + j;
				}
			}
		}
	}

	/* Collection Interval TLV*/
	struct tlv_metric_collection_interval *p1 = (struct tlv_metric_collection_interval *)calloc(1, sizeof(struct tlv_metric_collection_interval));
	p1->tlv_type = MAP_TLV_METRIC_COLLECTION_INTERVAL;
	p1->collection_interval = 1000;

	cmdu_data->num_tlvs = 2;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	cmdu_data->tlvs[0] = (uint8_t *)p;
	cmdu_data->tlvs[1] = (uint8_t *)p1;

        prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

static void prepare_apstats_response(struct cmdu_cstruct *rec_cmdu)
{
	char radio_id[18] = "11:12:13:14:15:16";
	char bssid[18] = "21:22:23:24:25:26";
	char sta_id[18] = "31:32:33:34:35:36";
	uint32_t j;
	struct cmdu_cstruct *cmdu_data;

        cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));
	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_AP_METRICS_RESPONSE;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	/* Radio Metrics TLV */
	struct tlv_radio_metrics *p2 = (struct tlv_radio_metrics *)calloc(1, sizeof(struct tlv_radio_metrics));
	p2->tlv_type = MAP_TLV_RADIO_METRICS;
	hwaddr_aton(radio_id, p2->radio_id);
	p2->noise = 61;
	p2->transmit = 62;
	p2->receive_self = 63;
	p2->receive_other = 64;

	/* AP Metrics TLV */
#if 0
// ToDo: need to fill properly
	struct tlv_ap_metrics *p = (struct tlv_ap_metrics *)calloc(1, sizeof(struct tlv_ap_metrics));
	p->tlv_type = MAP_TLV_AP_METRICS;
	hwaddr_aton(bssid, p->bssid);
	p->is_ac_be = 1;
	p->is_ac_bk = 1;
	p->is_ac_vi = 1;
	p->is_ac_vo = 1;
#endif

	/* Extended AP Metrics TLV */
	struct tlv_ap_ext_metrics *p1 = (struct tlv_ap_ext_metrics *)calloc(1, sizeof(struct tlv_ap_ext_metrics));
	p1->tlv_type = MAP_TLV_AP_EXTENDED_METRICS;
	hwaddr_aton(bssid, p1->bssid);
	p1->uni_bytes_sent = 1100;
	p1->uni_bytes_recv = 1200;
	p1->multi_bytes_sent = 1300;
	p1->multi_bytes_recv = 1400;
	p1->bro_bytes_sent = 1500;
	p1->bro_bytes_recv = 1600;

	/* Associated STA Link Metrics TLV */
	struct tlv_assoc_sta_link_metrics *p3 = (struct tlv_assoc_sta_link_metrics *)calloc(1, sizeof(struct tlv_assoc_sta_link_metrics));
	p3->tlv_type = MAP_TLV_ASSOCIATED_STA_LINK_METRICS;
	hwaddr_aton(sta_id, p3->addr);
	p3->bssid_nr = 1;
	p3->sta_link_metrics_bssid = calloc(p3->bssid_nr, sizeof(*p3->sta_link_metrics_bssid));
	for(j=0; j<p3->bssid_nr; j++) {
		hwaddr_aton(bssid, p3->sta_link_metrics_bssid[j].bssid);
		p3->sta_link_metrics_bssid[j].time_delta = 500;
		p3->sta_link_metrics_bssid[j].dl_mac_data_rate = 500;
		p3->sta_link_metrics_bssid[j].ul_mac_data_rate = 500;
		p3->sta_link_metrics_bssid[j].ul_rcpi = 200;
	}

	/* Associated STA Traffic Stats TLV*/
	struct tlv_assoc_sta_traffic_stats *p4 = (struct tlv_assoc_sta_traffic_stats *)calloc(1, sizeof(struct tlv_assoc_sta_traffic_stats));
	p4->tlv_type = MAP_TLV_ASSOCIATED_STA_TRAFFIC_STATS;
	hwaddr_aton(sta_id, p4->addr);
	p4->bytes_sent = 4000;
	p4->bytes_received = 4000;
	p4->packets_sent = 4000;
	p4->packets_received = 4000;
	p4->tx_packets_err = 4000;
	p4->rx_packets_err = 4000;

	/*Associated STA Extended Link Metrics TLV*/
	struct tlv_assoc_sta_ext_link_metric *p5 = (struct tlv_assoc_sta_ext_link_metric *)calloc(1, sizeof(struct tlv_assoc_sta_ext_link_metric));
	p5->tlv_type = MAP_TLV_ASSOCIATED_STA_EXT_LINK_METRICS;
	hwaddr_aton(sta_id, p5->mac);
	p5->nbr_bssid = 1;
	p5->data = calloc(p5->nbr_bssid, sizeof(*p5->data));
	for(j=0; j<p5->nbr_bssid; j++) {
		hwaddr_aton(bssid, p5->data[j].bssid);
		p5->data[j].last_data_dwn_rate = 2100;
		p5->data[j].last_data_up_rate = 2100;
		p5->data[j].uti_recv = 2100;
		p5->data[j].uti_transmit = 2100;
	}

	cmdu_data->num_tlvs = 5;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	//cmdu_data->tlvs[0] = (uint8_t *)p;
	cmdu_data->tlvs[0] = (uint8_t *)p1;
	cmdu_data->tlvs[1] = (uint8_t *)p2;
	cmdu_data->tlvs[2] = (uint8_t *)p3;
	cmdu_data->tlvs[3] = (uint8_t *)p4;
	cmdu_data->tlvs[4] = (uint8_t *)p5;

	prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

static void prepare_backhaul_response(struct cmdu_cstruct *rec_cmdu)
{
	struct cmdu_cstruct *cmdu_data;
	cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));
	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_BACKHAUL_STA_CAPABILITY_REPORT;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	struct tlv_backhaul_sta_radio_cap *p = (struct tlv_backhaul_sta_radio_cap *)calloc(1, sizeof(struct tlv_backhaul_sta_radio_cap));
	p->tlv_type = MAP_TLV_BACKHAUL_STA_RADIO_CAPABILITY;
	char radio[18] = "11:12:13:14:15:16";
	char bk_addr[18] = "41:42:43:44:45:46";
	hwaddr_aton(radio, p->radio_id);
	p->mac_include = 1;
	if (p->mac_include)
		hwaddr_aton(bk_addr, p->mac);


	cmdu_data->num_tlvs = 1;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	cmdu_data->tlvs[0] = (uint8_t *)p;

	prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

static void prepare_operating_channel_response(struct cmdu_cstruct *rec_cmdu)
{
	struct cmdu_cstruct *cmdu_data;
	cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));

	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_OPERATING_CHANNEL_REPORT;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	struct tlv_oper_ch_report *p = (struct tlv_oper_ch_report *)calloc(1, sizeof(struct tlv_oper_ch_report));
	p->tlv_type = MAP_TLV_OPERATING_CHANNEL_REPORT;
	char radio_id[18] = "11:12:13:14:15:16";
	hwaddr_aton(radio_id, p->radio_id);
	p->op_ch_op_class_nr = 4;
	uint8_t j = 0;
	p->op_ch_op_class = calloc(p->op_ch_op_class_nr, sizeof(*p->op_ch_op_class));
	for(j=0; j<p->op_ch_op_class_nr; j++) {
		p->op_ch_op_class[j].op_class = 111;
		p->op_ch_op_class[j].channel = 112;
	}
	p->curr_tx_power = 113;

	cmdu_data->num_tlvs = 1;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	cmdu_data->tlvs[0] = (uint8_t *)p;

	prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

static void prepare_unassoc_sta_response(struct cmdu_cstruct *rec_cmdu)
{
	uint8_t i = 0;
	struct cmdu_cstruct *cmdu_data;

	cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));
	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_UNASSOC_STA_LINK_METRIC_RESPONSE;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	struct tlv_unassoc_sta_link_metrics_resp *p =
		(struct tlv_unassoc_sta_link_metrics_resp *)
		calloc(1, sizeof(struct tlv_unassoc_sta_link_metrics_resp));

	p->tlv_type = MAP_TLV_UNASSOCIATED_STA_LINK_METRICS_RESPONSE;
	p->sta_nr = 4;
	p->unassoc_sta_link_metrics_sta = calloc(p->sta_nr, sizeof(*p->unassoc_sta_link_metrics_sta));
	char sta_addr[4][18] = {"AA:AA:AA:AA:AA:AA", "BB:BB:BB:BB:BB:BB", "CC:CC:CC:CC:CC:CC", "DD:DD:DD:DD:DD:DD"};
	for(i=0; i<p->sta_nr; i++) {
		hwaddr_aton(sta_addr[i], p->unassoc_sta_link_metrics_sta[i].addr);
		p->unassoc_sta_link_metrics_sta[i].ul_rcpi = 88;
	}

	cmdu_data->num_tlvs = 1;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	cmdu_data->tlvs[0] = (uint8_t *)p;

	prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

static void prepare_scanres_channel_response(struct cmdu_cstruct *rec_cmdu)
{
	struct cmdu_cstruct *cmdu_data;
	cmdu_data = (struct cmdu_cstruct *)calloc(1, sizeof(struct cmdu_cstruct));
	if (!cmdu_data) {
		return;
	}

	cmdu_data->message_type = CMDU_CHANNEL_SCAN_REPORT;
	memcpy(cmdu_data->origin, rec_cmdu->origin, 6);
	cmdu_data->message_id = rec_cmdu->message_id;
	strcpy(cmdu_data->intf_name, rec_cmdu->intf_name);

	struct tlv_ch_scan_res *p = (struct tlv_ch_scan_res *)calloc(1, sizeof(struct tlv_ch_scan_res));
	p->tlv_type = MAP_TLV_CHANNEL_SCAN_RES;
	char radio_id[18] = "11:12:13:14:15:16";
	hwaddr_aton(radio_id, p->radio_id);
	p->op_class = 151;
	p->ch = 151;
	p->scan_res = 0x00;
	p->time_len = 0;
	p->timestamp = NULL;
	p->utilization = 151;
	p->noise = 151;
	p->nbr_neighbors = 1;
	p->neighbor_data = calloc(p->nbr_neighbors, sizeof(*p->neighbor_data));
	uint16_t j = 0;
	for (j = 0; j < p->nbr_neighbors; j++) {
		char bssid[18] = "21:22:23:24:25:26";
		hwaddr_aton(bssid, p->neighbor_data[j].bssid);
		p->neighbor_data[j].ssid = (char *) calloc(20, sizeof(char));
		strcpy(p->neighbor_data[j].ssid, "iopsys_test");
		p->neighbor_data[j].ssid_len = strlen(p->neighbor_data[j].ssid);
		p->neighbor_data[j].signal_strength = 100 + j;
		p->neighbor_data[j].ch_bandwidth = calloc(10, sizeof(char));
		strcpy(p->neighbor_data[j].ch_bandwidth, "30");
		p->neighbor_data[j].ch_band_len = strlen(p->neighbor_data[j].ch_bandwidth);
		p->neighbor_data[j].bss_load_elm_present = 1;
		if (p->neighbor_data[j].bss_load_elm_present == 1) {
			p->neighbor_data[j].ch_util = 100 + j;
			p->neighbor_data[j].sta_count = 200;
		}
	}

	p->agg_scan_duration = 10;
	p->scan_type = 0;
	cmdu_data->num_tlvs = 1;
	cmdu_data->tlvs = (uint8_t **)calloc(cmdu_data->num_tlvs, sizeof(uint8_t *));
	cmdu_data->tlvs[0] = (uint8_t *)p;

	prepare_and_send_cmdu_cstruct(cmdu_data);
	map_free_cmdu(cmdu_data);
}

void send_dummy_responses_for_testing()
{
	// prepare a dummy req cmdu for testing
	// using multicast destination address for now
	struct cmdu_cstruct c = {
		.message_version = 0,
		.message_type = 3,
		.message_id = 12345,
		.intf_name = "br-lan",
		.origin = {0x00, 0x22, 0x07, 0xA2, 0xF0, 0x52},
		.relay_indicator = 0,
		.num_tlvs = 0,
	};

	prepare_topology_response(&c);
	prepare_ap_capability_response(&c);
	prepare_apstats_response(&c);
	prepare_backhaul_response(&c);
	prepare_operating_channel_response(&c);
	prepare_unassoc_sta_response(&c);
	prepare_scanres_channel_response(&c);
}

void test_cmdu_conversion(uint16_t type, uint16_t mid, char *tlv_str)
{
	struct cmdu_cstruct *c = NULL;
	uint8_t source[6] = {0x02, 0x22, 0x07, 0x9f, 0xb7, 0xfe};
	uint8_t	tlvs[500];

	_strtob(tlvs, tlv_str);
	c = map_build_cmdu(type, mid, NULL, source, tlvs);
	if (!c) {
		printf("no cmdu returned!!!");
		return;
	}

	printf("type|%d|, mid |%d|, ingress(%s) tlv_present|%d|\n",
	       c->message_type, c->message_id,\
	       c->intf_name, c->num_tlvs);

	for (size_t i = 0; i < c->num_tlvs; i++) {
		printf("cmdu{0x%x} => TLV_index{%zd} -> ", type, i);
		uint16_t tlv_len = 0;
		uint8_t *tlv = map_put_tlv_cstruct(c->tlvs[i], &tlv_len);

		dump_tlv_bytes(tlv, tlv_len);

		free(tlv);
		printf("\n");
	}

	map_free_cmdu(c);
}

void test_cmdu_conversion_vector()
{
	size_t i;

	char *tlv_str[] = {
		"8b002e0022076b17510d7300007600007900007400007700007a00007500007800007b00007d00008000008100008200008d00070022076b17511e8b00100022076b1752035100005300005400008d00070022076b17521e000000",
		"030036002207765c5403002207765c5501050a002207765c5500026464002207765c5601030a002207765c5600000b0b002207765c5400010006000c002207765c54e8d8d1f0b59407000d002207765c540022079fb7fc00000000",
		NULL
	};
	uint16_t cmdu_vec[] = {
		3,
		32774,
		0
	};

	for (i = 0; tlv_str[i] != NULL; i++) {
		test_cmdu_conversion(cmdu_vec[i], 1234, tlv_str[i]);
	}
}

void test_converting_timestamp_tlv(void)
{
        struct tlv_timestamp tlv;
        uint8_t *tlv_stream = NULL;
        uint16_t tlv_len = 0;

        // fill the tlv with correct info
        tlv.tlv_type = MAP_TLV_TIMESTAMP;
        tlv.time = "2020-08-24T15:20:16+01:00";
        tlv.time_len = strlen(tlv.time);

        tlv_stream = map_put_tlv_cstruct((uint8_t *) &tlv, &tlv_len);

	dump_tlv_bytes(tlv_stream, tlv_len);

	free(tlv_stream);
}

int main()
{
	test_converting_timestamp_tlv();
	test_cmdu_conversion_vector();
	test_converting_tlv_cstructure_to_stream();
	send_dummy_responses_for_testing();

	return 0;
}
