/*
 * map_module.h - multi-ap module(s) registration API header.
 *
 * Copyright (C) 2020 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef MAP_MODULE_H
#define MAP_MODULE_H

#include <stdint.h>
#include <stdbool.h>
#include <linux/types.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef uint8_t map_module_id_t[32];

enum map_role {
	MAP_ROLE_AGENT = 0x1,
	MAP_ROLE_CONTROLLER = 0x2,
	MAP_ROLE_TOPOLOGYD = 0x4
};

enum map_profile {
	MAP_PROFILE_1 = 0x1,
	MAP_PROFILE_2 = 0x2,
};

/**
 * @brief map-agent and map-controller will implement the multiap CMDU handler
 * function. Based on the cmdu_type, appropriate action will be taken by
 * map-agents and map-controller. 'msg' points to the TLVs buffer, as defined
 * in the multiap specification and in that order. 'msglen' is length of the 'msg'.
 */
typedef int (*cmdu_handler_t)(uint16_t cmdu_type, void *msg, size_t msglen);


/**
 * @brief map-modules have to use the below structure to register them with map-plugin
 *        map-modules have to fill the appropriate values and then convert the structure
 *        into hex-string, which it needs to pass as 'data' while doing registration
 *        using ubus method 'register' of map-plugin ubus '1905.multiap' object.
 *        More details of registration available in README.
 */
struct map_module {
	map_module_id_t uid;         /**< handle to module object */
	uint8_t sign[32];
	uint32_t role;               /**< bitmap of MAP_ROLE_* */
	enum map_profile profile;    /**< one of MAP_PROFILE_* */
	cmdu_handler_t map_cmdu_rx;  /**< multiap CMDU handler function */

	/* cmdu type filter */
	int num_cmdu_types;          /**< wildcard is '0', meaning all */
	uint16_t cmdutype[64];       /**< present if num_cmdu_types != 0 */
} __packed;

#ifdef __cplusplus
}
#endif

#endif /* MAP_MODULE_H */
